// Automatically generated file. Do not edit.
#ifndef AVOGADRO_STATIC_QTPLUGINS_H
#define AVOGADRO_STATIC_QTPLUGINS_H
#include <QtCore/QtPlugin>

Q_IMPORT_PLUGIN(ThreeDMolFactory)
Q_IMPORT_PLUGIN(AlchemyFactory)
Q_IMPORT_PLUGIN(AlignToolFactory)
Q_IMPORT_PLUGIN(apbsFactory)
Q_IMPORT_PLUGIN(ApplyColorsFactory)
Q_IMPORT_PLUGIN(AutoOptFactory)
Q_IMPORT_PLUGIN(BondCentricFactory)
Q_IMPORT_PLUGIN(BondingFactory)
Q_IMPORT_PLUGIN(CentroidFactory)
Q_IMPORT_PLUGIN(ConfigurePythonFactory)
Q_IMPORT_PLUGIN(ConstraintsExtensionFactory)
Q_IMPORT_PLUGIN(CoordinateEditorFactory)
Q_IMPORT_PLUGIN(CopyPasteFactory)
Q_IMPORT_PLUGIN(Cp2kInputFactory)
Q_IMPORT_PLUGIN(CrystalFactory)
Q_IMPORT_PLUGIN(CrystalSceneFactory)
Q_IMPORT_PLUGIN(CustomElementsFactory)
Q_IMPORT_PLUGIN(DipoleFactory)
Q_IMPORT_PLUGIN(EditorFactory)
Q_IMPORT_PLUGIN(FetchPDBFactory)
Q_IMPORT_PLUGIN(FocusFactory)
Q_IMPORT_PLUGIN(ForcefieldFactory)
Q_IMPORT_PLUGIN(GamessInputFactory)
Q_IMPORT_PLUGIN(HydrogensFactory)
Q_IMPORT_PLUGIN(ImportPQRFactory)
Q_IMPORT_PLUGIN(NucleicInputFactory)
Q_IMPORT_PLUGIN(InsertFragmentFactory)
Q_IMPORT_PLUGIN(InsertPeptideFactory)
Q_IMPORT_PLUGIN(LabelFactory)
Q_IMPORT_PLUGIN(LabelEditorFactory)
Q_IMPORT_PLUGIN(LammpsInputFactory)
Q_IMPORT_PLUGIN(LineFormatInputFactory)
Q_IMPORT_PLUGIN(ManipulatorFactory)
Q_IMPORT_PLUGIN(MeasureToolFactory)
Q_IMPORT_PLUGIN(MolecularPropertiesFactory)
Q_IMPORT_PLUGIN(NavigatorFactory)
Q_IMPORT_PLUGIN(NetworkDatabasesFactory)
Q_IMPORT_PLUGIN(OpenBabelFactory)
Q_IMPORT_PLUGIN(OpenMMInputFactory)
Q_IMPORT_PLUGIN(PlayerToolFactory)
Q_IMPORT_PLUGIN(PLYFactory)
Q_IMPORT_PLUGIN(POVRayFactory)
Q_IMPORT_PLUGIN(PropertyTablesFactory)
Q_IMPORT_PLUGIN(ResetViewFactory)
Q_IMPORT_PLUGIN(SelectFactory)
Q_IMPORT_PLUGIN(SelectionFactory)
Q_IMPORT_PLUGIN(SpaceGroupFactory)
Q_IMPORT_PLUGIN(SurfacesFactory)
Q_IMPORT_PLUGIN(OrbitalsFactory)
Q_IMPORT_PLUGIN(SVGFactory)
Q_IMPORT_PLUGIN(TemplateToolFactory)
Q_IMPORT_PLUGIN(VibrationsFactory)
Q_IMPORT_PLUGIN(VRMLFactory)
Q_IMPORT_PLUGIN(SpectraFactory)
Q_IMPORT_PLUGIN(PlotConformerFactory)
Q_IMPORT_PLUGIN(PlotPdfFactory)
Q_IMPORT_PLUGIN(PlotXrdFactory)
Q_IMPORT_PLUGIN(YaehmopFactory)
Q_IMPORT_PLUGIN(commandsFactory)
Q_IMPORT_PLUGIN(QuantumInputFactory)
Q_IMPORT_PLUGIN(ScriptChargesFactory)
Q_IMPORT_PLUGIN(ScriptFileFormatsFactory)
Q_IMPORT_PLUGIN(PackageInstallerFactory)
Q_IMPORT_PLUGIN(SymmetryFactory)
Q_IMPORT_PLUGIN(SymmetrySceneFactory)
Q_IMPORT_PLUGIN(BallStickFactory)
Q_IMPORT_PLUGIN(CartoonsFactory)
Q_IMPORT_PLUGIN(CloseContactsFactory)
Q_IMPORT_PLUGIN(ForceFactory)
Q_IMPORT_PLUGIN(LicoriceFactory)
Q_IMPORT_PLUGIN(SurfaceRenderFactory)
Q_IMPORT_PLUGIN(NonCovalentFactory)
Q_IMPORT_PLUGIN(VanDerWaalsFactory)
Q_IMPORT_PLUGIN(WireframeFactory)
Q_IMPORT_PLUGIN(OverlayAxesFactory)

void initAvogadroPluginResources() {
  Q_INIT_RESOURCE(aligntool);
  Q_INIT_RESOURCE(autoopt);
  Q_INIT_RESOURCE(bondcentrictool);
  Q_INIT_RESOURCE(editor);
  Q_INIT_RESOURCE(label);
  Q_INIT_RESOURCE(manipulator);
  Q_INIT_RESOURCE(measuretool);
  Q_INIT_RESOURCE(navigator);
  Q_INIT_RESOURCE(playertool);
  Q_INIT_RESOURCE(selectiontool);
  Q_INIT_RESOURCE(template);
}


#endif // AVOGADRO_STATIC_QTPLUGINS_H
